'use strict';

const fs = require('fs');
const util = require('util');
const cloneDeep = require('clone-deep');
const utils = require('punch-utils');

const ScraperManager = require('../index');
const scraperManager = new ScraperManager({
  scraperConfig: {
    strategy: {
      name: ScraperManager.STRATEGIES.HTTP
    }
  },
  proxyManagerConfig: {
    proxyConfig: {
      // Logic for proxies to blacklist themselves.
      blacklistIf: (state) => {
        // Return if > 70% of the distinct URLs tested failed and at least
        // 5 have been attempted.
        return ((state.failed.distinctUrlCount/state.tried.distinctUrlCount) > .7) &&
            state.tried.distinctUrlCount > 5;
      }
    }
  }
});

scraperManager.start();

let topSites = [];
for (let i = 0; i < 20; i++) {
  topSites.push(`http://www.alexa.com/topsites/global;${i}`);
}

scraperManager.scrape(topSites, {
  onUpdate: (status, update) => {
    console.log(util.inspect(update, false, 5, true));
    status = cloneDeep(status);
    utils.object.forEach(status.links, (link) => {
      // Delete for logging purposes.
      link.result = 'Removed for logging purposes.';
    });
    status.successful = status.successful.sort();
    status.failed = status.failed.sort();
    console.log(util.inspect(status, false, 5, true));
  }
})
.then(results => {
  return results.map(result => {
    let links = [];
    let $ = result.html.getParsed();
    let $listings = $('.site-listing');
    $listings.each(function() {
      let formatted = {};
      let $listing = $(this);
      let $link = $listing.find('.desc-paragraph a');
      formatted.ranking = Number($listing.find('.count').text().trim());
      formatted.url = $link.text().trim();
      formatted.alexa_url = `http://www.alexa.com${$link.attr('href').trim()}`;
      let $description = $listing.find('.description');
      formatted.description = $description.text().replace('…More', '').trim();
      links.push(formatted);
    });
    return links.sort((a, b) => a.ranking - b.ranking);
  })
  .reduce((arr, list) => {
    // return [...arr, ...list];
    return arr.concat(list);
  }, []);
})
.then(response => {
  console.log('Success!');
  console.log(response);
})
.catch(err => {
  console.log('ERROR', err);
});