'use strict';

const request = require('request-promise');
const deepFreeze = require('deep-freeze-strict');
const AppErr = require('punch-error');
const ERROR = require('../constants').ERROR;

const ScrapedResult = require('../../scraped-result');
const SocksAgent = require('socks5-http-client/lib/Agent');

const DEFAULTS = deepFreeze({
  METHOD: 'get',
  TIMEOUT: 30 * 1000
});


module.exports.scrape = function(link, config) {
  config = config || {};
  config.method = config.method || DEFAULTS.METHOD;
  config.url = link;
  config.timeout = config.timeout || DEFAULTS.TIMEOUT;
  if (config.proxy && config.proxy.getIP) {
    var ipPort = config.proxy.getIP();
    var ip = ipPort.split(':')[0];
    var port = ipPort.split(':')[1];
    var type = config.proxy.getType();

    if (type == 'socks5' || type == 'socks4') {
      config.agentClass = SocksAgent;
      config.agentOptions = {
        socksHost: ip,
        socksPort: port
      };
      delete config.proxy;
    }
    else if (type == 'https'){
      config.proxy = 'https://' + ipPort;
    }
    else{
      config.proxy = 'http://' + ipPort;
    }
  }

  return request(config)
  .then(html => {
    return new ScrapedResult(html);
  })
  .catch((error) => {
    console.log('HTTP strategy caught error.');
    return AppErr.reject(error, ERROR.REQUEST_ERROR);
  })
};